%%%%%%%%%%%%%%%%% Example 10.6 %%%%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ---- Observer-controller design ----
%
clear
disp('Example 10.6')
%-- (a) find control gain F and observer gain L ----
Ts = 0.1;
A = [0.81   -0.23   -0.045
     0.09    0.98   -0.0023
     0.005   0.10    1     ];
B = [0.09 0.0047 0.00016]';
C = [1 3.5 3]; D = 0;  
G = ss(A,B,C,D,Ts)            % build plant as LTI object
disp('******>'), pause
p_s = [0.75 0.8 0.9]';        % desired system poles 
F = place(A,B,p_s)            % controller gains
p_o = [0.5 0.6 0.75]';        % desired observer poles
L = place(A',C',p_o)'         % observer gains
%-- (b) build observer-controller according to (10.5) and (10.6) ---
A_oc = A-B*F-L*C              % obsv-cont system matrix
disp('******>'), pause
Goc = ss(A_oc,-L,-F,0,Ts)     % build obsrv-cont as LTI object
disp('******>'), pause
Goc_poles = pole(Goc)         % controller poles are all real
Goc_zeros = tzero(Goc)
disp('******>'), pause
figure
bode(Goc),grid                % Figure 10.3
Gpc = G*Goc;                  % controller & plant in cascade
disp('******>'), pause
Gcl = feedback(Gpc,1,-1)      % CL system with non-unity DC gain 
disp('******>'), pause
cl_loop_poles = pole(Gcl)     % CL system poles
%-- (c) pre-gain for unity DC gain -------------------------
lfg = dcgain(Gcl)             % low-frequency gain
N = 1/lfg                     % normalization constant
disp('******>'), pause
%-- incorporate N in series with closed-loop system transfer function
T_ref = N*Gcl                 % ..unity dc gain
disp('******>'), pause

t = [0:Ts:4];                 % column vector of time
r = 0*t;                      % zero reference input
z0 = [1 -0.75 0.5 0 0 0]      % initial condition vector
[y,t,z] = lsim(T_ref,r,t,z0); % IC simulation
disp('******>'), pause
% plot system states and observer states
figure
plot(t,z(:,1:3),'o',t,z(:,4),'+',t,z(:,5),'x',t,z(:,6),'^'), grid
legend('x_1','x_2','x_3','x_1hat','x_2hat','x_3hat')
title('Ex 10.6: Observer-Controller Design')
xlabel('Time (s)')
ylabel('Amplitude')
text(0.40,0.85,'x_1')
text(0.33,-0.50,'x_2')
text(0.07,0.50,'x_3')
figure
[ys,t,z] = step(T_ref,4);     % compute step resp to 4 sec.
stem(t,ys,':*'); grid
title('Ex 10.6: Step Response')
xlabel('Time (s)')
ylabel('Amplitude')
[M0,tp,tr,ts,ess] = kstats(t,ys,1) % check time-domain performance
%%%%%%%%%%