%%%%%%%%%%% Comprehensive Problem 3.2 %%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inverted Pendulum
% System has 4 states, 1 input, and 2 outputs
% 4 states : pendulum's angular position and velocity, and
%            cart's position and velocity
% 1 input  : motor voltage (u)
% 2 outputs: pendulum's angular position (theta)
%            cart's position (x) 

disp('CP3.2: Inverted Pendulum with Angle Feedback')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part A : Load model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear; close all

% Load continuous and discrete-time systems
if ~exist('stick.mat'), 
    dstick
elseif exist('stick.mat') == 2,
    load stick
end

Ts = 0.01;  % in dstick already
disp('*******>'); pause; disp(' ');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part B : Kp(z) proportional-and-derivative control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Extract model from control input to pendulum angle')
disp('Transfer Function Gp(z) from u to theta')
Gp = tf(dnump,ddenp,Ts)
disp('*******>');pause

disp('Display pole-zero map')
ucircle
hold on
pzmap(Gp)
title('Pole-zero map from input to pendulam angle')
hold off
disp('*******>');pause

disp('PD controller for pendulum angle')
KP = 1.5; 
KD = 0.2;
KpPD = tf(KP*[(1+KD/Ts) -KD/Ts],[1 0],Ts) % PD controller
disp('*******>');pause

disp('Closed-loop control system')
Gp_CL = feedback(Gp*KpPD,1,1,1) % CL angle control
disp('*******>');pause

disp('Closed-loop system poles')
pole(Gp_CL)
disp('There is a pole outside the unit circle')
disp('*******>');pause

disp('End of Comprehensive Problem CP3.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%