%%%%%%%%%%%%%%%%% Example 8.1(c) %%%%%%%%%%%%%%%%
%   Discrete-Time Control Problems using        %
%       MATLAB and the Control System Toolbox   %
%   by J.H. Chow, D.K. Frederick, & N.W. Chbat  %
%         Brooks/Cole Publishing Company        %
%                September 2002                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   ----  Closed-loop reference & disturbance step responses  ----
%
clear
disp('Example 8.1(c)')

Ts = 0.1;			                % sampling period
Gp_s = tf(4,conv([2 1],[0.5 1]));   % continuous process
H_s = tf(1,[0.05 1]);		        % continuous sensor
Gp_z = c2d(Gp_s,Ts,'zoh');	        % discrete plant (no sensor)
HGp_z = c2d(H_s*Gp_s,Ts,'zoh');     % discrete plant + sensor

KP = 1.032;	                        % proportional gain for 20 pct overshoot
time = 0:Ts:8;                      % define time vector for plots  

%------ reference input - CL transfer fcn
T_ref = KP*Gp_z/(1 + HGp_z*KP)      % build CL system with reference input
T_ref = minreal(T_ref);             % remove any pole-zero cancellations
y_ref = step(T_ref,time);           % CL step response to reference input

%----- disturbance input - CL transfer fcn
T_dist = Gp_z/(1 + HGp_z*KP)        % build CL system with disturbance input
T_dist = minreal(T_dist);           % remove any pole-zero cancellations
y_dist = step(T_dist,time);         % CL step response to disturbance input

%-------- plot responses
figure; 
plot(time,y_ref,'o',time,y_dist,'+') % plot both responses
xlabel('Time (s)')
ylabel('Amplitude')
title(['Example 8.1(c) - Resp to step ref (o) and disturbance (+) inputs'])
grid
hold off
text('Units','norm','Pos',[0.6,0.84],'Str','reference')
text('Units','norm','Pos',[0.7,0.74],'Str','disturbance')
text('Units','norm','Pos',[0.6,0.20],'Str',['KP = ',num2str(KP)])

% steady state
y_ref_ss = y_ref(length(time))     % final value for reference input 
y_dist_ss = y_dist(length(time))   % final value for disturbance input

%----- compute prcent OS
pct_OS = 100 * (max(y_ref)/y_ref_ss - 1)
%%%%%%%%%%